"""Base class for all analyzers to inherit as they create indictable events."""

from __future__ import absolute_import

import pdb
import sys
from time import time as NOW

from .IndictQueues import IndictQueues

sys.modules[__name__].HP = True

class Epriv(object):
    """Give Indictable subclasses a separate namespace for their private
    data.  It keeps the "top level" of an Indictable clean.  All Epriv really
    does is create a __dict__, as inheriting directly from object won't."""

    def __init__(self, base, **kwds):
        """Just give it a __dict__"""
        self.__dict__ = kwds
        self.__EnclosingBase__ = base   # Not sure what I'll do with this...

    def __repr__(self):
        """Don't override me."""
        s = ""
        for k, v in self.__dict__:
            s += "%-10s: %s\n" % (k, v) # ENH: pretty print module?
        return s

class Indictable(object):
    """Every analyzer must inherit and extend this base class to create
    a RAS event, filling in physloc at a minimum.  Then it should invoke
    the Indict() method for processing."""

    def __init__(self):
        """Instantiate common attributes."""
        self.physloc = None       # trap below
        self.TS = 0               # timestamp of event arrival/expiration
        self.CPH = 60             # Count per hour, should never trigger
        self.IndictedCB = None    # When Indicter decides FRU has gone bad
        self.DeactivateCB = None  # When Indicter decides to offline a FRU
        self.IndexStr = ""        # Short text used in debug output
        self.E = Epriv(self.__class__.__name__) # subclass (event) private data

    def __cmp__(self, other):
        """Not needed in general.  Override if you do."""
        raise NotImplementedError

    def __eq__(self, other):
        """Support of dictionary operations.  Must be overridden."""
        raise NotImplementedError

    def __hash__(self):
        """Support of dictionary operations.  Must be overridden."""
        raise NotImplementedError

    def Diagonal(self):             # FIXME: make this a get property???
        """String representation of an indicted event.  Can be overridden."""
        return repr(self.physloc)

    def Indict(self, IndictedCB = None, DeactivateCB = None):
        """Add myself to the Incoming list."""
        if not self.physloc or not len(self.E.__dict__):
            raise AttributeError("Incomplete physloc or private data")
        self.IndictedCB = IndictedCB
        self.DeactivateCB = DeactivateCB
        # Give preference to the analyzer's timestamp, if set (sometimes
        # for debugging or test reasons).
        if not self.TS:
            self.TS = int(NOW())
        IndictQueues.AddIndictable(self)
