"""Provide command line parsing and options for all modules."""

from __future__ import absolute_import	# Forces python 2.5

import pdb
import logging
import logging.handlers
from optparse import OptionParser

class Globals(object):
    """Provide command-line (and other) options."""

    LOGFILE = "/var/log/hp/hprasd"

    opts = None
    Stats = [ ]		# Statistics call chain
    ResetStats = [ ]

    @classmethod
    def parse_cmdline(cls, prog_args):
        """Only call this once to process command line arguments."""
        if cls.opts:
            return        # Already been called
        parser = OptionParser(                      # handles --version
            usage = "%prog [-f] [-l] [-v[v[v]]] [--version] [--help]",
            version = "Version 0.31 24 March 2009",
            description = "Reliability, Availability, and Serviceability " +
                "daemon")
        parser.add_option(
            "-f", "--fg", dest = "fg",
            action = "store_true", default = False,
            help = "Foreground (don't become a daemon)")
        parser.add_option(
            "-l", "--logfile", dest = "log2file",
            action = "store_true", default = True,
            help = "use (rotating) log files at %s" % cls.LOGFILE)
        parser.add_option(
            "-v", "--verbose", dest = "verbose",
            action = "count", default = 0,
            help = "Verbosity (1v == WARNING, 2v's == INFO, 3v's == DEBUG)")
        optstuple = parser.parse_args(prog_args)
        opts = optstuple[0]
        cls.opts = opts

        # Inter-option relationships
        if opts.fg:
            if not opts.verbose:
                opts.verbose = 1
        else:
            opts.log2file = True

        # This helper does nothing if there's already a root logger.  Otherwise
        # it creates a StreamHandler with a default Formatter and adds it
        # to the root logger.  So do it first.
        if opts.fg:
            logging.basicConfig(
                datefmt="%H:%M:%S",
                format='%(asctime)s %(module)s: %(message)s')
        rootlogger = logging.getLogger()    # Needed later, get it now

        # With no argument, it gets the root logger.  Extend/create one
        if opts.log2file:
            # Doesn't take the "format" keyword like basicConfig so
            # grind it out.
            handler = logging.handlers.RotatingFileHandler(
                cls.LOGFILE, maxBytes=1024 * 100, backupCount=5)
            formatter = logging.Formatter(
                '%(asctime)s %(module)s: %(message)s',
                datefmt="%Y-%m-%d %H:%M:%S")
            handler.setFormatter(formatter)
            rootlogger.addHandler(handler)

        if opts.verbose > 2:
            rootlogger.setLevel(logging.DEBUG)
        elif opts.verbose > 1:
            rootlogger.setLevel(logging.INFO)
        elif opts.verbose > 0:
            rootlogger.setLevel(logging.WARNING)
        else:
            rootlogger.setLevel(logging.ERROR)
