# EventManager - Organize gdk/gobject initialization and export helpers
"""DBus transactions are registered with the Gtk event loop.  This module
   MUST be imported first as it sets up that loop."""

import pdb
import gobject
import logging
import os
import signal
import sys

from RASconfig import Globals

sys.modules[__name__].HP = True

_REMloop = None

def MainLoop(start=True):
    """Call this once from the outermost module after all submodules
       have finished initializing."""

    if start:
        if not _REMloop.is_running():
            logging.warning("starting main loop, PID %d", os.getpid())
            _REMloop.run()  # ENH: does this have return values?
        else:
            logging.warning("main loop is already running, you rude boy")
    else:
        if _REMloop.is_running():
            logging.warning("stop main loop")
            _REMloop.quit()
        else:
            logging.warning("main loop is already stopped, you silly boy")
    return

def EventRemove(ID):
    """Remove a Gtk event/callback (like a timeout).  No errors are thrown."""
    try:
        gobject.source_remove(ID)
    finally:
        pass

_idles = { }

def _IdleCB(actualCB):
    """Hook to invoke actual idle callback with latest data."""
    unused_ID, userdata = _idles[actualCB]
    del _idles[actualCB]
    return actualCB(userdata)  # False means no resched

def IdleAdd(callback, newcbdata=None):
    """Add a callback that's invoked during gtk loop idle operation.
    Returns the event ID.  Duplicate additions are detected; callback
    data will be updated."""
    if not callable(callback):
        raise TypeError
    if callback in _idles:
        ID, unused_oldcbdata = _idles[callback]
    else:
        ID = gobject.idle_add(_IdleCB, callback)
    _idles[callback] = (ID, newcbdata)
    return ID

_timeouts = { }

def _TimeoutCB(actualCB):
    """Hook to invoke actual timeout callback with latest data."""
    unused_ID, userdata = _timeouts[actualCB]
    del _timeouts[actualCB]
    return actualCB(userdata)  # False means no resched

def TimeoutAdd(seconds, callback, newcbdata=None, replace=True):
    """Add a relative timeout, returning the event ID.  If replace is True,
    an existing timeout (by callback) will be cleared first."""
    if not callable(callback) or seconds <= 0:
        raise TypeError
    if callback in _timeouts:
        ID, unused_cbdata = _timeouts[callback]
        if not replace:
            return ID
        EventRemove(ID)
    ID = gobject.timeout_add_seconds(seconds, _TimeoutCB, callback)
    _timeouts[callback] = (ID, newcbdata)
    return ID

def _KeyboardInterrupt(cbdata=None):
    """Idle event loop callback chained from the SIGINT handler."""
    try:
        raw = raw_input("\n(d)ebug, (p)rint stats, (r)eset stats, or (q)uit ? ")
    except (EOFError, KeyboardInterrupt):		# Two in a row
        print
        raw = "q"
    if "q" in raw:
        MainLoop(False)
        return False
    elif "d" in raw:
        pdb.set_trace()
    elif "p" in raw:
        allstats = ""
        for c in Globals.Stats:   # Each one ends in linefeed
            allstats += c()
        print allstats
    elif "r" in raw:
        for c in Globals.ResetStats:
            c()
    print "Returning to main loop...\n"
    return False

# Don't handle things out of this signal handler, defer them.

def _sighandler(signum, unused_frame):
    """Handles SIGHUP | QUIT | INT | TERM.  Don't call this directly."""
    if Globals.opts.fg and signum == signal.SIGINT:
        IdleAdd(_KeyboardInterrupt)	# Defer
    else:
        logging.warning("Received signal %d", signum)
        MainLoop(False)

# This must be done before a DBus Connection, Bus Instance, or Proxy
# object is created.  Other methods behind the DBus module will then
# use this main loop.  BTW, threads_init can also be found in gtk.gdk
# but that always throws error messages about no DISPLAY, or no RANDR.

def Stats():
    """Invoked from the chain for DBus method 'GetStats'"""
    s = ""
    for cb in sorted(_timeouts.keys()):
        s += "REM: Timeout: %s.%s()\n" % (
            cb.__module__, cb.__func__.__name__)
    for cb in sorted(_idles.keys()):
        s += "REM: Idle:    %s.%s()\n" % (
            cb.__module__, cb.__func__.__name__)
    return s

def setup():
    """Instantiate DBus bindings and initialize gtk main loop.  Should be
    called only from package __init__.py."""
    global _REMloop

    try:
        from dbus.mainloop.glib import DBusGMainLoop
    except ImportError, unused_e:
        raise ImportError("Install the 'dbus-1-python' RPM (version >= 0.81)")

    DBusGMainLoop(set_as_default = True)
    _REMloop = gobject.MainLoop()

    # NO SIGALRM as it's probably used by gtk event loop
    signal.signal(signal.SIGHUP,  _sighandler)
    signal.signal(signal.SIGINT,  _sighandler)
    signal.signal(signal.SIGQUIT, _sighandler)
    signal.signal(signal.SIGTERM, _sighandler)
    Globals.Stats.append(Stats)
    return True
