"""Support for daemonizing and PID file creation"""

from __future__ import absolute_import, with_statement

import pdb
import logging
import os
import os.path
import sys

sys.modules[__name__].HP = True

PIDFILE = "/var/run/hprasd.pid"

def PIDfile_create():
    """Called once when the PID is known.  Returns True/False."""
    if os.path.isfile(PIDFILE):     # is something running?
        pid = 0
        with open(PIDFILE, "rt") as f:
            data = f.read()
            if data:
                pid = int(data)
        if pid:                     # ping it
            try:
                os.kill(pid, 0)
                logging.critical("PID %d (see %s) appears to be a copy",
                    pid, PIDFILE)
                return False
            except:
                pass
        try:
            os.unlink(PIDFILE)
        except:
            pass

    with open(PIDFILE, "wt") as f:
        f.write(str(os.getpid()))
    return True

def PIDfile_destroy():
    """Called once when the application is exiting."""
    try:
        os.remove(PIDFILE)
    except:
        pass

def daemonize(newstdin='/dev/null',
              newstdout='/dev/null',
              newstderr='/dev/null'):
    """Python version of daemon(2) system call."""
    try:
        pid = os.fork()
        if pid > 0:     # Original process "A" (fork caller)
            sys.exit(0)
    except OSError, e:
        logging.exception("daemonize fork #1 failed:", e.strerror)
        return False
    # I am childA, the child of process A.  Now dissociate from its inherited
    # (console) environment so a new child won't pick it up.
    os.chdir("/")
    os.umask(0)
    os.setsid()
    try:
        pid = os.fork()
        if pid > 0:     # I am childA, my work is done
            sys.exit(0)
    except OSError, e:
        logging.exception("daemonize fork #2 failed:", e.strerror)
        return False
    # This process, childOfChildA, is now daemonized, redirect stdin/out/err.
    for f in sys.stdout, sys.stderr:
        f.flush()
    fdstdin  = file(newstdin, 'r')
    fdstdout = file(newstdout, 'a+')
    fdstderr = file(newstderr, 'a+', 0)
    os.dup2(fdstdin.fileno(),  sys.stdin.fileno())
    os.dup2(fdstdout.fileno(), sys.stdout.fileno())
    os.dup2(fdstderr.fileno(), sys.stderr.fileno())
    return True
